@@ -2,7 +2,7 @@ adjust_em_sel_val <- function(
     data_observed,
     data_validation) {
   if (!all(data_observed$confounders %in% data_validation$confounders)) {
-    stop("All confounders in observed data must be present in validation data.")
+    stop("All confounders in observed data must be present in validation data.", call. = FALSE)
   }
 
   if (is.null(data_validation$misclassified_exposure)) {
@@ -11,7 +11,8 @@ adjust_em_sel_val <- function(
         "This function is adjusting for a misclassified exposure.",
         "\n",
         "Validation data must have a true and misclassified exposure specified."
-      )
+      ),
+      call. = FALSE
     )
   }
   if (is.null(data_validation$selection)) {
@@ -20,7 +21,8 @@ adjust_em_sel_val <- function(
         "This function is adjusting for selection bias.",
         "\n",
         "Validation data must have a selection indicator column specified."
-      )
+      ),
+      call. = FALSE
     )
   }
 
@@ -351,7 +353,10 @@ adjust_em_sel_coef <- function(
       })
     }
   } else if (len_c > 3) {
-    stop("This function is currently not compatible with >3 confounders.")
+    stop(
+      "This function is currently not compatible with >3 confounders.",
+      call. = FALSE
+    )
   }
 
   return(final)
@@ -453,7 +458,7 @@ adjust_emc_sel <- function(
 #'
 #' # Using x_model_coefs and s_model_coefs -------------------------------------
 #' adjust_em_sel(
-#'   df_observed,
+#'   data_observed = df_observed,
 #'   x_model_coefs = c(-2.78, 1.62, 0.58, 0.34),
 #'   s_model_coefs = c(0.04, 0.18, 0.92, 0.05)
 #' )
@@ -475,14 +480,12 @@ adjust_em_sel <- function(
     x_model_coefs = NULL,
     s_model_coefs = NULL,
     level = 0.95) {
-  if (
-    (!is.null(data_validation) && !is.null(x_model_coefs)) ||
-      (is.null(data_validation) && is.null(x_model_coefs))
-  ) {
-    stop("One of: 1. data_validation or 2. (x_model_coefs & s_model_coefs) must be non-null.")
-  }
-  data <- data_observed$data
+  check_inputs2(
+    data_validation,
+    list(x_model_coefs, s_model_coefs)
+  )
 
+  data <- data_observed$data
   xstar <- data[, data_observed$exposure]
   y <- data[, data_observed$outcome]
 
