@@ -1,42 +1,159 @@
-#' Adust for uncontrolled confounding, outcome misclassification, and selection
-#' bias.
-#'
-#' @description
-#' `r lifecycle::badge("deprecated")`
-#'
-#' `adjust_uc_omc_sel()` was renamed to `adjust_uc_om_sel()`
-#' @keywords internal
-#'
-#' @export
-adjust_uc_omc_sel <- function(
+adjust_uc_om_sel_val <- function(
     data_observed,
-    u_model_coefs = NULL,
-    y_model_coefs = NULL,
-    u0y1_model_coefs = NULL,
-    u1y0_model_coefs = NULL,
-    u1y1_model_coefs = NULL,
-    s_model_coefs,
-    level = 0.95) {
-  lifecycle::deprecate_warn(
-    "1.5.3", "adjust_uc_omc_sel()", "adjust_uc_om_sel()"
+    data_validation) {
+  if (!all(data_observed$confounders %in% data_validation$confounders)) {
+    stop(
+      "All confounders in observed data must be present in validation data.",
+      call. = FALSE
+    )
+  }
+
+  if (
+    length(data_validation$confounders) - length(data_observed$confounders) != 1
+  ) {
+    stop(
+      paste0(
+        "This function adjusts for unobserved confounding from one confounder.",
+        "\n",
+        "Validation data must have one more confounder than the observed data."
+      ),
+      call. = FALSE
+    )
+  }
+  if (is.null(data_validation$misclassified_outcome)) {
+    stop(
+      paste0(
+        "This function is adjusting for a misclassified outcome.",
+        "\n",
+        "Validation data must have a true and misclassified outcome specified."
+      ),
+      call. = FALSE
+    )
+  }
+  if (is.null(data_validation$selection)) {
+    stop(
+      paste0(
+        "This function is adjusting for selection bias.",
+        "\n",
+        "Validation data must have a selection indicator column specified."
+      ),
+      call. = FALSE
+    )
+  }
+
+  n <- nrow(data_observed$data)
+
+  df <- data.frame(
+    X = data_observed$data[, data_observed$exposure],
+    Ystar = data_observed$data[, data_observed$outcome]
   )
-  adjust_uc_om_sel(
-    data_observed,
-    u_model_coefs,
-    y_model_coefs,
-    u0y1_model_coefs,
-    u1y0_model_coefs,
-    u1y1_model_coefs,
-    s_model_coefs,
-    level
+  df <- bind_cols(
+    df,
+    data_observed$data %>%
+      select(all_of(data_observed$confounders))
   )
+
+  df_val <- data.frame(
+    X = data_validation$data[, data_validation$true_exposure],
+    Y = data_validation$data[, data_validation$true_outcome],
+    Ystar = data_validation$data[, data_validation$misclassified_outcome],
+    S = data_validation$data[, data_validation$selection]
+  )
+
+  uc <- setdiff(data_validation$confounders, data_observed$confounders)
+  df_val$U <- data_validation$data[, uc]
+  df_val <- bind_cols(
+    df_val,
+    data_validation$data %>%
+      select(all_of(data_observed$confounders))
+  )
+
+  force_match(
+    df$X,
+    df_val$X,
+    "Exposures from both datasets must both be binary or both be continuous."
+  )
+  force_binary(
+    df_val$U,
+    "Uncontrolled confounder in validation data must be a binary integer."
+  )
+  force_binary(
+    df$Ystar,
+    "Outcome in observed data must be a binary integer."
+  )
+  force_binary(
+    df_val$Ystar,
+    "Misclassified outcome in validation data must be a binary integer."
+  )
+  force_binary(
+    df_val$Y,
+    "True outcome in validation data must be a binary integer."
+  )
+  force_binary(
+    df_val$S,
+    "Selection indicator in validation data must be a binary integer."
+  )
+
+  y_mod <- glm(Y ~ X + Ystar + . - U - S,
+    family = binomial(link = "logit"),
+    data = df_val
+  )
+
+  y_mod_coefs <- coef(y_mod)
+  y_pred <- y_mod_coefs[1]
+
+  for (i in 2:length(y_mod_coefs)) {
+    var_name <- names(y_mod_coefs)[i]
+    y_pred <- y_pred + df[[var_name]] * y_mod_coefs[i]
+  }
+
+  df$Ypred <- rbinom(n, 1, plogis(y_pred))
+
+  u_mod <- glm(U ~ X + Y,
+    family = binomial(link = "logit"),
+    data = df_val
+  )
+
+  u_mod_coefs <- coef(u_mod)
+  u_pred <- u_mod_coefs[1]
+
+  for (i in 2:length(u_mod_coefs)) {
+    var_name <- names(u_mod_coefs)[i]
+    var_name <- gsub("Y", "Ypred", var_name) # col Y is not in df
+    u_pred <- u_pred + df[[var_name]] * u_mod_coefs[i]
+  }
+
+  df$Upred <- rbinom(n, 1, plogis(u_pred))
+
+  s_mod <- glm(S ~ X + Ystar + . - Y - U,
+    family = binomial(link = "logit"),
+    data = df_val
+  )
+
+  s_mod_coefs <- coef(s_mod)
+  s_pred <- s_mod_coefs[1]
+
+  for (i in 2:length(s_mod_coefs)) {
+    var_name <- names(s_mod_coefs)[i]
+    s_pred <- s_pred + df[[var_name]] * s_mod_coefs[i]
+  }
+
+  df$Spred <- plogis(s_pred)
+
+  suppressWarnings({
+    final <- glm(
+      Ypred ~ X + Upred + . - Ystar - Spred,
+      family = binomial(link = "logit"),
+      weights = (1 / df$Spred),
+      data = df
+    )
+  })
+
+  return(final)
 }
 
 
-
-# bias adjust with u_model_coefs and y_model_coefs
-
-uc_om_sel_single <- function(
+adjust_uc_om_sel_coef_single <- function(
     data_observed,
     u_model_coefs,
     y_model_coefs,
@@ -47,10 +164,12 @@ uc_om_sel_single <- function(
   len_c <- length(confounders)
   len_u_coefs <- length(u_model_coefs)
   len_y_coefs <- length(y_model_coefs)
+  len_s_coefs <- length(s_model_coefs)
 
   x <- data[, data_observed$exposure]
   ystar <- data[, data_observed$outcome]
 
+  force_binary(ystar, "Outcome must be a binary integer.")
   force_len(
     len_u_coefs,
     3,
@@ -67,6 +186,14 @@ uc_om_sel_single <- function(
       "Length should equal 3 + number of confounders."
     )
   )
+  force_len(
+    len_s_coefs,
+    3 + len_c,
+    paste0(
+      "Incorrect length of S model coefficients. ",
+      "Length should equal 3 + number of confounders."
+    )
+  )
 
   u1_0 <- u_model_coefs[1]
   u1_x <- u_model_coefs[2]
@@ -218,16 +345,17 @@ uc_om_sel_single <- function(
       )
     })
   } else if (len_c > 3) {
-    stop("This function is currently not compatible with >3 confounders.")
+    stop(
+      "This function is currently not compatible with >3 confounders.",
+      call. = FALSE
+    )
   }
 
   return(final)
 }
 
 
-# bias adjust with multinomial coefs
-
-uc_om_sel_multinom <- function(
+adjust_uc_om_sel_coef_multinom <- function(
     data_observed,
     u1y0_model_coefs,
     u0y1_model_coefs,
@@ -240,10 +368,12 @@ uc_om_sel_multinom <- function(
   len_u0y1_coefs <- length(u0y1_model_coefs)
   len_u1y0_coefs <- length(u1y0_model_coefs)
   len_u1y1_coefs <- length(u1y1_model_coefs)
+  len_s_coefs <- length(s_model_coefs)
 
   x <- data[, data_observed$exposure]
   ystar <- data[, data_observed$outcome]
 
+  force_binary(ystar, "Outcome must be a binary integer.")
   force_len(
     len_u1y0_coefs,
     3 + len_c,
@@ -539,19 +669,60 @@ uc_om_sel_multinom <- function(
       )
     })
   } else if (len_c > 3) {
-    stop("This function is currently not compatible with >3 confounders.")
+    stop(
+      "This function is currently not compatible with >3 confounders.",
+      call. = FALSE
+    )
   }
 
   return(final)
 }
 
 
+#' Adust for uncontrolled confounding, outcome misclassification, and selection
+#' bias.
+#'
+#' @description
+#' `r lifecycle::badge("deprecated")`
+#'
+#' `adjust_uc_omc_sel()` was renamed to `adjust_uc_om_sel()`
+#' @keywords internal
+#'
+#' @export
+adjust_uc_omc_sel <- function(
+    data_observed,
+    u_model_coefs = NULL,
+    y_model_coefs = NULL,
+    u0y1_model_coefs = NULL,
+    u1y0_model_coefs = NULL,
+    u1y1_model_coefs = NULL,
+    s_model_coefs,
+    level = 0.95) {
+  lifecycle::deprecate_warn(
+    "1.5.3", "adjust_uc_omc_sel()", "adjust_uc_om_sel()"
+  )
+  adjust_uc_om_sel(
+    data_observed,
+    u_model_coefs,
+    y_model_coefs,
+    u0y1_model_coefs,
+    u1y0_model_coefs,
+    u1y1_model_coefs,
+    s_model_coefs,
+    level
+  )
+}
+
+
 #' Adust for uncontrolled confounding, outcome misclassification, and selection
 #' bias.
 #'
 #' `adjust_uc_om_sel` returns the exposure-outcome odds ratio and
 #' confidence interval, adjusted for uncontrolled confounding, outcome
-#' misclassificaiton, and selection bias. Two different options for the bias
+#' misclassificaiton, and selection bias.
+#'
+#' Bias adjustment can be performed by inputting either a validation dataset or
+#' the necessary bias parameters. Two different options for the bias
 #' parameters are availale here: 1) parameters from separate models
 #' of *U* and *Y* (`u_model_coefs` and `y_model_coefs`)
 #' or 2) parameters from a joint model of *U* and *Y*
@@ -570,6 +741,13 @@ uc_om_sel_multinom <- function(
 #'
 #' @param data_observed Object of class `data_observed` corresponding to the
 #' data to perform bias analysis on.
+#' @param data_validation Object of class `data_validation` corresponding to
+#' the validation data used to adjust for bias in the observed data. Here, the
+#' validation data should have data for the same variables as in the observed
+#' data, plus data for: 1) the true and misclassified outcome corresponding
+#' to the observed outcome in `data_observed`, 2) the confounder missing in
+#' `data_observed`, 3) a selection indicator representing whether the
+#' observation in `data_validation` was selected in `data_observed`.
 #' @param u_model_coefs The regression coefficients corresponding to the model:
 #' \ifelse{html}{\out{logit(P(U=1)) = &alpha;<sub>0</sub> + &alpha;<sub>1</sub>X + &alpha;<sub>2</sub>Y, }}{\eqn{logit(P(U=1)) = \alpha_0 + \alpha_1 X + \alpha_2 Y, }}
 #' where *U* is the binary unmeasured confounder, *X* is the
@@ -624,15 +802,31 @@ uc_om_sel_multinom <- function(
 #' confidence interval as the vector: (lower bound, upper bound).
 #'
 #' @examples
-#' df <- data_observed(
+#' df_observed <- data_observed(
 #'   data = df_uc_om_sel,
 #'   exposure = "X",
 #'   outcome = "Ystar",
 #'   confounders = c("C1", "C2", "C3")
 #' )
+#'
+#' # Using validation data -----------------------------------------------------
+#' df_validation <- data_validation(
+#'   data = df_uc_om_sel_source,
+#'   true_exposure = "X",
+#'   true_outcome = "Y",
+#'   confounders = c("C1", "C2", "C3", "U"),
+#'   misclassified_outcome = "Ystar",
+#'   selection = "S"
+#' )
+#'
+#' adjust_uc_om_sel(
+#'   data_observed = df_observed,
+#'   data_validation = df_validation
+#' )
+#'
 #' # Using u_model_coefs, y_model_coefs, s_model_coefs -------------------------
 #' adjust_uc_om_sel(
-#'   df,
+#'   data_observed = df_observed,
 #'   u_model_coefs = c(-0.32, 0.59, 0.69),
 #'   y_model_coefs = c(-2.85, 0.71, 1.63, 0.40, -0.85, 0.22),
 #'   s_model_coefs = c(0.00, 0.74, 0.19, 0.02, -0.06, 0.02)
@@ -640,7 +834,7 @@ uc_om_sel_multinom <- function(
 #'
 #' # Using u1y0_model_coefs, u0y1_model_coefs, u1y1_model_coefs, s_model_coefs
 #' adjust_uc_om_sel(
-#'   df,
+#'   data_observed = df_observed,
 #'   u1y0_model_coefs = c(-0.20, 0.62, 0.01, -0.08, 0.10, -0.15),
 #'   u0y1_model_coefs = c(-3.28, 0.63, 1.65, 0.42, -0.85, 0.26),
 #'   u1y1_model_coefs = c(-2.70, 1.22, 1.64, 0.32, -0.77, 0.09),
@@ -660,56 +854,40 @@ uc_om_sel_multinom <- function(
 
 adjust_uc_om_sel <- function(
     data_observed,
+    data_validation = NULL,
     u_model_coefs = NULL,
     y_model_coefs = NULL,
     u0y1_model_coefs = NULL,
     u1y0_model_coefs = NULL,
     u1y1_model_coefs = NULL,
-    s_model_coefs,
+    s_model_coefs = NULL,
     level = 0.95) {
-  data <- data_observed$data
-  x <- data[, data_observed$exposure]
-  ystar <- data[, data_observed$outcome]
-  confounders <- data_observed$confounders
-
-  len_c <- length(confounders)
-  len_s_coefs <- length(s_model_coefs)
-
-  force_binary(ystar, "Outcome must be a binary integer.")
-
-  force_len(
-    len_s_coefs,
-    3 + len_c,
-    paste0(
-      "Incorrect length of S model coefficients. ",
-      "Length should equal 3 + number of confounders."
-    )
+  check_inputs3(
+    input1 = data_validation,
+    input2 = list(u_model_coefs, y_model_coefs, s_model_coefs),
+    input3 = list(
+      u1y0_model_coefs,
+      u0y1_model_coefs,
+      u1y1_model_coefs,
+      s_model_coefs
+    ),
+    ignore = s_model_coefs
   )
 
-  # check that user correctly specified bias parameters
-  if (
-    !(
-      (is.null(u_model_coefs) && is.null(y_model_coefs)) ||
-        ((is.null(u1y0_model_coefs) && is.null(u0y1_model_coefs) &&
-          is.null(u1y1_model_coefs)))
+  if (!is.null(data_validation)) {
+    final <- adjust_uc_om_sel_val(
+      data_observed,
+      data_validation
     )
-  ) {
-    stop(
-      "In addition to s_model_coefs, bias parameters must be specified for:\n
-       1) u_model_coefs and y_model_coefs or\n
-       2) u1y0_model_coefs, u0y1_model_coefs, and u1y1_model_coefs."
-    )
-  }
-
-  if (!is.null(y_model_coefs)) {
-    final <- uc_om_sel_single(
+  } else if (!is.null(y_model_coefs)) {
+    final <- adjust_uc_om_sel_coef_single(
       data_observed = data_observed,
       u_model_coefs = u_model_coefs,
       y_model_coefs = y_model_coefs,
       s_model_coefs = s_model_coefs
     )
   } else if (!is.null(u1y0_model_coefs)) {
-    final <- uc_om_sel_multinom(
+    final <- adjust_uc_om_sel_coef_multinom(
       data_observed = data_observed,
       u1y0_model_coefs = u1y0_model_coefs,
       u0y1_model_coefs = u0y1_model_coefs,
