@@ -1,68 +1,144 @@
-#' Adust for uncontrolled confounding and selection bias.
-#'
-#' `adjust_uc_sel` returns the exposure-outcome odds ratio and confidence
-#' interval, adjusted for uncontrolled confounding and exposure
-#' misclassificaiton.
-#'
-#' Values for the regression coefficients can be applied as
-#' fixed values or as single draws from a probability
-#' distribution (ex: `rnorm(1, mean = 2, sd = 1)`). The latter has
-#' the advantage of allowing the researcher to capture the uncertainty
-#' in the bias parameter estimates. To incorporate this uncertainty in the
-#' estimate and confidence interval, this function should be run in loop across
-#' bootstrap samples of the dataframe for analysis. The estimate and
-#' confidence interval would then be obtained from the median and quantiles
-#' of the distribution of odds ratio estimates.
-#'
-#' @param data_observed Object of class `data_observed` corresponding to the
-#' data to perform bias analysis on.
-#' @param u_model_coefs The regression coefficients corresponding to the model:
-#' \ifelse{html}{\out{logit(P(U=1)) = &alpha;<sub>0</sub> + &alpha;<sub>1</sub>X + &alpha;<sub>2</sub>Y + &alpha;<sub>2+j</sub>C<sub>j</sub>, }}{\eqn{logit(P(U=1)) = \alpha_0 + \alpha_1 X + \alpha_2 Y + \alpha_{2+j} C_j, }}
-#' where *U* is the binary unmeasured
-#' confounder, *X* is the exposure, *Y* is the outcome, *C*
-#' represents the vector of measured confounders (if any), and *j*
-#' corresponds to the number of measured confounders. The number of parameters
-#' therefore equals 3 + *j*.
-#' @param s_model_coefs The regression coefficients corresponding to the model:
-#' \ifelse{html}{\out{logit(P(S=1)) = &beta;<sub>0</sub> + &beta;<sub>1</sub>X + &beta;<sub>2</sub>Y, }}{\eqn{logit(P(S=1)) = \beta_0 + \beta_1 X + \beta_2 Y, }}
-#' where *S* represents binary selection, *X* is the exposure,
-#' and *Y* is the outcome. The number of parameters therefore equals 3.
-#' @param level Value from 0-1 representing the full range of the confidence
-#' interval. Default is 0.95.
-#
-#' @return A list where the first item is the odds ratio estimate of the
-#' effect of the exposure on the outcome and the second item is the
-#' confidence interval as the vector: (lower bound, upper bound).
-#'
-#' @examples
-#' df <- data_observed(
-#'   data = df_uc_sel,
-#'   exposure = "X",
-#'   outcome = "Y",
-#'   confounders = c("C1", "C2", "C3")
-#' )
-#' adjust_uc_sel(
-#'   df,
-#'   u_model_coefs = c(-0.19, 0.61, 0.72, -0.09, 0.10, -0.15),
-#'   s_model_coefs = c(-0.01, 0.92, 0.94)
-#' )
-#'
-#' @import dplyr
-#' @importFrom magrittr %>%
-#' @importFrom stats binomial
-#' @importFrom stats glm
-#' @importFrom stats lm
-#' @importFrom stats qnorm
-#' @importFrom stats plogis
-#' @importFrom rlang .data
-#'
-#' @export
+adjust_uc_sel_val <- function(
+    data_observed,
+    data_validation) {
+  if (!all(data_observed$confounders %in% data_validation$confounders)) {
+    stop(
+      "All confounders in observed data must be present in validation data.",
+      call. = FALSE
+    )
+  }
 
-adjust_uc_sel <- function(
+  if (
+    length(data_validation$confounders) - length(data_observed$confounders) != 1
+  ) {
+    stop(
+      paste0(
+        "This function adjusts for unobserved confounding from one confounder.",
+        "\n",
+        "Validation data must have one more confounder than the observed data."
+      ),
+      call. = FALSE
+    )
+  }
+  if (is.null(data_validation$selection)) {
+    stop(
+      paste0(
+        "This function is adjusting for selection bias.",
+        "\n",
+        "Validation data must have a selection indicator column specified."
+      ),
+      call. = FALSE
+    )
+  }
+
+  n <- nrow(data_observed$data)
+
+  df <- data.frame(
+    X = data_observed$data[, data_observed$exposure],
+    Y = data_observed$data[, data_observed$outcome]
+  )
+  df <- bind_cols(
+    df,
+    data_observed$data %>%
+      select(all_of(data_observed$confounders))
+  )
+
+  if (all(df$Y %in% 0:1)) {
+    y_binary <- TRUE
+  } else {
+    y_binary <- FALSE
+  }
+
+  df_val <- data.frame(
+    X = data_validation$data[, data_validation$true_exposure],
+    Y = data_validation$data[, data_validation$true_outcome],
+    S = data_validation$data[, data_validation$selection]
+  )
+
+  uc <- setdiff(data_validation$confounders, data_observed$confounders)
+  df_val$U <- data_validation$data[, uc]
+  df_val <- bind_cols(
+    df_val,
+    data_validation$data %>%
+      select(all_of(data_observed$confounders))
+  )
+
+  force_match(
+    df$X,
+    df_val$X,
+    "Exposures from both datasets must both be binary or both be continuous."
+  )
+  force_match(
+    df$Y,
+    df_val$Y,
+    "Outcomes from both datasets must both be binary or both be continuous."
+  )
+  force_binary(
+    df_val$U,
+    "Uncontrolled confounder in validation data must be a binary integer."
+  )
+  force_binary(
+    df_val$S,
+    "Selection indicator in validation data must be a binary integer."
+  )
+
+  u_mod <- glm(U ~ X + Y + . - S,
+    family = binomial(link = "logit"),
+    data = df_val
+  )
+
+  u_mod_coefs <- coef(u_mod)
+  u_pred <- u_mod_coefs[1]
+
+  for (i in 2:length(u_mod_coefs)) {
+    var_name <- names(u_mod_coefs)[i]
+    u_pred <- u_pred + df[[var_name]] * u_mod_coefs[i]
+  }
+
+  df$Upred <- rbinom(n, 1, plogis(u_pred))
+
+  s_mod <- glm(S ~ X + Y,
+    family = binomial(link = "logit"),
+    data = df_val
+  )
+
+  s_mod_coefs <- coef(s_mod)
+  s_pred <- s_mod_coefs[1]
+
+  for (i in 2:length(s_mod_coefs)) {
+    var_name <- names(s_mod_coefs)[i]
+    s_pred <- s_pred + df[[var_name]] * s_mod_coefs[i]
+  }
+
+  df$Spred <- plogis(s_pred)
+
+  if (y_binary) {
+    suppressWarnings({
+      final <- glm(
+        Y ~ X + Upred + . - Spred,
+        family = binomial(link = "logit"),
+        weights = (1 / df$Spred),
+        data = df
+      )
+    })
+  } else {
+    suppressWarnings({
+      final <- lm(
+        Y ~ X + Upred + . - Spred,
+        weights = (1 / df$Spred),
+        data = df
+      )
+    })
+  }
+
+  return(final)
+}
+
+
+adjust_uc_sel_coef <- function(
     data_observed,
     u_model_coefs,
-    s_model_coefs,
-    level = 0.95) {
+    s_model_coefs) {
   data <- data_observed$data
   n <- nrow(data)
   confounders <- data_observed$confounders
@@ -260,7 +336,130 @@ adjust_uc_sel <- function(
       })
     }
   } else if (len_c > 3) {
-    stop("This function is currently not compatible with >3 confounders.")
+    stop(
+      "This function is currently not compatible with >3 confounders.",
+      call. = FALSE
+    )
+  }
+
+  return(final)
+}
+
+
+#' Adust for uncontrolled confounding and selection bias.
+#'
+#' `adjust_uc_sel` returns the exposure-outcome odds ratio and confidence
+#' interval, adjusted for uncontrolled confounding and exposure
+#' misclassificaiton.
+#'
+#' Bias adjustment can be performed by inputting either a validation dataset or
+#' the necessary bias parameters. Values for the bias parameters
+#' can be applied as fixed values or as single draws from a probability
+#' distribution (ex: `rnorm(1, mean = 2, sd = 1)`). The latter has
+#' the advantage of allowing the researcher to capture the uncertainty
+#' in the bias parameter estimates. To incorporate this uncertainty in the
+#' estimate and confidence interval, this function should be run in loop across
+#' bootstrap samples of the dataframe for analysis. The estimate and
+#' confidence interval would then be obtained from the median and quantiles
+#' of the distribution of odds ratio estimates.
+#'
+#' @param data_observed Object of class `data_observed` corresponding to the
+#' data to perform bias analysis on.
+#' @param data_validation Object of class `data_validation` corresponding to
+#' the validation data used to adjust for bias in the observed data. Here, the
+#' validation data should have data for the same variables as in the observed
+#' data, plus data for the confounder missing in `data_observed`. There
+#' should also be a selection indicator representing whether the observation in
+#' `data_validation` was selected in `data_observed`.
+#' @param u_model_coefs The regression coefficients corresponding to the model:
+#' \ifelse{html}{\out{logit(P(U=1)) = &alpha;<sub>0</sub> + &alpha;<sub>1</sub>X + &alpha;<sub>2</sub>Y + &alpha;<sub>2+j</sub>C<sub>j</sub>, }}{\eqn{logit(P(U=1)) = \alpha_0 + \alpha_1 X + \alpha_2 Y + \alpha_{2+j} C_j, }}
+#' where *U* is the binary unmeasured
+#' confounder, *X* is the exposure, *Y* is the outcome, *C*
+#' represents the vector of measured confounders (if any), and *j*
+#' corresponds to the number of measured confounders. The number of parameters
+#' therefore equals 3 + *j*.
+#' @param s_model_coefs The regression coefficients corresponding to the model:
+#' \ifelse{html}{\out{logit(P(S=1)) = &beta;<sub>0</sub> + &beta;<sub>1</sub>X + &beta;<sub>2</sub>Y, }}{\eqn{logit(P(S=1)) = \beta_0 + \beta_1 X + \beta_2 Y, }}
+#' where *S* represents binary selection, *X* is the exposure,
+#' and *Y* is the outcome. The number of parameters therefore equals 3.
+#' @param level Value from 0-1 representing the full range of the confidence
+#' interval. Default is 0.95.
+#
+#' @return A list where the first item is the odds ratio estimate of the
+#' effect of the exposure on the outcome and the second item is the
+#' confidence interval as the vector: (lower bound, upper bound).
+#'
+#' @examples
+#' df_observed <- data_observed(
+#'   data = df_uc_sel,
+#'   exposure = "X",
+#'   outcome = "Y",
+#'   confounders = c("C1", "C2", "C3")
+#' )
+#' # Using validation data -----------------------------------------------------
+#' df_validation <- data_validation(
+#'   data = df_uc_sel_source,
+#'   true_exposure = "X",
+#'   true_outcome = "Y",
+#'   confounders = c("C1", "C2", "C3", "U"),
+#'   selection = "S"
+#' )
+#'
+#' adjust_uc_sel(
+#'   data_observed = df_observed,
+#'   data_validation = df_validation
+#' )
+#'
+#' # Using u_model_coefs and s_model_coefs -------------------------------------
+#' adjust_uc_sel(
+#'   data_observed = df_observed,
+#'   u_model_coefs = c(-0.19, 0.61, 0.72, -0.09, 0.10, -0.15),
+#'   s_model_coefs = c(-0.01, 0.92, 0.94)
+#' )
+#'
+#' @import dplyr
+#' @importFrom magrittr %>%
+#' @importFrom stats binomial
+#' @importFrom stats glm
+#' @importFrom stats lm
+#' @importFrom stats qnorm
+#' @importFrom stats plogis
+#' @importFrom rlang .data
+#'
+#' @export
+
+adjust_uc_sel <- function(
+    data_observed,
+    data_validation = NULL,
+    u_model_coefs = NULL,
+    s_model_coefs = NULL,
+    level = 0.95) {
+  check_inputs2(
+    data_validation,
+    list(u_model_coefs, s_model_coefs)
+  )
+
+  data <- data_observed$data
+  x <- data[, data_observed$exposure]
+  y <- data[, data_observed$outcome]
+
+  if (all(y %in% 0:1)) {
+    y_binary <- TRUE
+  } else {
+    y_binary <- FALSE
+  }
+
+  if (!is.null(data_validation)) {
+    final <- adjust_uc_sel_val(
+      data_observed,
+      data_validation
+    )
+  } else if (!is.null(u_model_coefs)) {
+    final <- adjust_uc_sel_coef(
+      data_observed,
+      u_model_coefs,
+      s_model_coefs
+    )
   }
 
   est <- summary(final)$coef[2, 1]
